;; Generated package description from load-dir.el  -*- no-byte-compile: t -*-
(define-package "load-dir" "0.0.5.0.20220207.124832" "Load all Emacs Lisp files in a given directory" '((cl-lib "0.5")) :commit "6e2bb9e4df79c16531e15f7551c33bc31ed8a2e3" :url "https://elpa.gnu.org/packages/load-dir.html" :maintainer '("Teodor Zlatanov" . "tzz@lifelogs.com") :keywords '("lisp" "files" "convenience"))
